#!/usr/bin/env python3

import getopt
import sys
import random

import qrcode
from PIL import Image, ImageDraw, ImageFilter

debug=0

# c:断片化セルサイズ, d:分割数
cellsize=6  # ← 2だと読める、verson=12なら6だと読めない
divide=2  # 細かすぎると、エラー補正で補正されて全部の絵が揃わなくても読み取れる

try:
    opts, args = getopt.getopt(sys.argv[1:], "D:c:d:", ["debug=", "cellsize=", "divide="])
except getopt.GetoptError as err:
    print(err)
    sys.exit(1)
for o, a in opts:
    if o in ("-D", "--debug"):
        debug = int(a)
        print("debug=%d"%debug)
    elif o in ("-c", "--cellsize"):
        cellsize=int(a)
        print("cellsize=%d"%cellsize)
    elif o in ("-d", "--divide"):
        divide = int(a)
        print("divide=%d"%divide)
    else:
        assert False, "wrong option"

qr = qrcode.QRCode(
    #error_correction=qrcode.constants.ERROR_CORRECT_H, # L(7%), default M(15%), Q(25%), H(30%)
    box_size=4, # default=10
    border=4    # min=4
)

if debug>1:
    qr.add_data('これは、ペンです😃')
else:
    for line in sys.stdin:
        qr.add_data(line)

try:
    qr.make()
except qrcode.exceptions.DataOverflowError:
    print("Strings too long")
    exit(1)

# QR_CellSize = 4 * version + 17
qr.box_size=int( -(8.0/39.0)*qr.version + (382.0/39.0)+1) 
print("version=%d, box_size=%d, border=%d"%(qr.version, qr.box_size, qr.border))

qr_img = qr.make_image()
#qr_img = qr.make_image(fill_color="green", back_color="#2030e0")

if debug>0:
    qr_img.save('./qrcode_orig.png')

print(qr_img.size)

while (qr_img.size[0] // cellsize) % divide == 0:
    cellsize += 1

print("cellsize=%d, divide=%d" % (cellsize, divide))

mask=[]
draw=[]
for i in range(0,divide):
    mask += [Image.new("L", qr_img.size, 0)]
    draw += [ImageDraw.Draw(mask[i])]

#r=0
for y in range(0,qr_img.size[1],cellsize):
    for x in range(0,qr_img.size[0],cellsize):
        #r+=1
        r=random.randint(0,divide-1)
        draw[r%divide].rectangle((x, y, x+cellsize, y+cellsize), fill=255)

for i in range(0,divide):
    if debug>2:
        mask[i].save('mask'+str(i)+'.png')

    qr_s=qr_img.copy()
    qr_s = qr_s.convert("RGBA")
    qr_s.putalpha(mask[i])

    qr_s.save('qrcode'+str(i)+'.png')

