%{
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>

#include "book.h"

extern int yylex();
extern int yyerror(char []);
extern char *yytext;
extern int yyleng;
extern char *yyfpath;
%}

%union {
    char *cptr;
    int i;
    struct sketch *sptr;
    struct option *optr;
    struct action *aptr;
}

%token TAG
%token NAME
%token TEXT
%token OPTION
%token GOTO
%token END
%token LB
%token RB
%token SEMICOLON
%start s
%%

s   : book { printf("%s has been successfully parsed.\n", yyfpath); }
book : /* nothing */
        { printf("%s start reading.\n", yyfpath); }
     | book sketch

sketch : label LB text options RB { $<sptr>$ = add_sketch($<cptr>1, $<cptr>3, $<optr>4); }

label : NAME { if (find_sketch(yytext) != NULL) {
                    fprintf(stderr, "parser:the name [%s] already in use.\n", yytext);
                    exit(1);
                }
                $<cptr>$=strcreate(yytext);
             }

text : TEXT { $<cptr>$=strcreate(yytext); }

options : option action         { $<optr>$ = add_option($<cptr>1, $<aptr>2); }
        | options option action { struct option *o;
                                    for (o = $<optr>1; o->next != NULL; o=o->next) ;
                                    o->next = add_option($<cptr>2, $<aptr>3);
                                    $<optr>$ = $<optr>1;
                                }

option : OPTION { $<cptr>$=strcreate(yytext); }

action : goto tag SEMICOLON  { $<aptr>$=create_action($<i>1, $<cptr>2); }
       | end SEMICOLON       { $<aptr>$=create_action($<i>1, NULL); }
       | end tag SEMICOLON   { $<aptr>$=create_action($<i>1, $<cptr>2); }

goto : GOTO { $<i>$ = CMD_GOTO; }

tag : TAG { $<cptr>$ = strcreate(yytext); }

end : END { $<i>$ = CMD_END; }

