#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "book.h"

int set_debug_game(int);
int text_adventure(char *);
void display_text(struct sketch *);
void display_position(struct sketch *);
int select_options(struct option *);
int display_options(struct option *);
void help(void);
struct action *next_action(struct option *, int);
static void player_fail(char *, char *);

static int debug_game=0;
static struct sketch *spos;

static int moveon(char *);
static char *moveback(void);
#define RING_SIZE 256   // 前のスケッチに戻れる数
static char *footprint[RING_SIZE];
static int fptail=0, fppos=0;

extern int sslex(void);
extern void stexts (char *);

int set_debug_game(int d)
{
    return (debug_game=d);
}

int text_adventure(char *storepos)
{
    struct action *na;
    char *prev;
    if (!book)  return -1;
    if (storepos) {
        printf("\n\nGame is starting from [%s]... \n", storepos);
        if ((spos = find_sketch(storepos)) == NULL)
                player_fail("no label found", storepos);
    } else {
        spos=book;
        printf("\n\nGame is starting ... \n");
    }
    while (1) {
        printf("\n-----------------------\n");
        display_text(spos);
        if ((na = next_action( spos->opts, select_options(spos->opts))) == NULL ) {
            if ((prev = moveback())) {
                printf("move back to [%s]\n", prev);
                spos = find_sketch(prev); // エラー処理不要
                continue;
            }
            printf("no history back\n");
            continue;
        }
        switch (na->command) {
          case CMD_GOTO:
            if (debug_game) printf("goto:[%s]\n",na->to);
            moveon(spos->name);
            if ((spos = find_sketch(na->to)) == NULL)
                player_fail("text_advenure():no label found", na->to);
            break;
          case CMD_END:
            if (na->to) {
                // 終了状態の処理は、ここに記述
                printf("The game ended by status (%s)\n", na->to);
            } else {
                printf("The game is over.\n");
            }
            return 0; break;
          default:
            return 0; break;
        }
    }
}

void display_text(struct sketch *s)
{
    if (s==NULL) {
        player_fail("display_text():","NULL");
    }
    if (debug_game) display_position(s);
    stexts(s->text);
    sslex();
    printf("\n");
}

void display_position(struct sketch *s)
{
    if (fppos!=fptail)
        printf("(%s)->", footprint[ ((fppos-1) % RING_SIZE) ]);
    printf("[[%s]] ", s->name);
}

int select_options(struct option *opt)
{
    int i;
    int r=0;
    char ans[128];

    i=display_options(opt);
    while (r<1 || r>=i) {
        do {
            printf("which one?: ");
        } while (fgets(ans, 128, stdin)==NULL);
        r=atoi(ans);
        if (r == 0) {
            if (strncmp("quit", ans, 4)==0) { // 予期せぬ終了をさけるため、4文字チェック
                printf("Quiting the game. Bye.\n---SD.\n");
                exit(0);
            } else if (strncmp("exit", ans, 2)==0) { // 東に行くコマンドとの間違いを避けるため2文字チェック
                printf("Exiting the game on [%s]. See ya.\n---SD.\n", spos->name);
                exit(0);
            } else if (strncmp("pos", ans, 2)==0) {
                printf("You are now on [[%s]].\n", spos->name);
            } else if (strncmp("options", ans, 1)==0) {
                display_options(opt);
            } else if (debug_game && strncmp("prev", ans, 2)==0) {
                return -1;
            } else if (strncmp("help", ans, 1)==0) {
                help();
            } else if ( i == 2 ) { // 選択肢が1つしかない場合
                return 1;
            }
        }
    }
    return r;
}

int display_options(struct option *opt)
{
    int i;
    struct option *o;
    for (i=1,o=opt; o!=NULL; i++, o=o->next) {
        printf(" %2d) %s\n", i, o->mesg);
        if (debug_game>1)
            printf("<DEBUG>:command=%d, tag=%s\n", o->action->command, o->action->to);
    }
    return i;
}

struct action *next_action(struct option *opt, int s)
{
    struct option *r = opt;
    int i;
    if (s == -1) return NULL; // prev
    for (i=1; i<s; i++) r=r->next;
    if (debug_game>1)
        printf("<DEBUG>:next_action=%d, option=\"%s\", command=%d, tag=%s\n",
                i, r->mesg, r->action->command, r->action->to);

    return r->action;
}

static void player_fail(char *mesg, char *info)
{
    fprintf(stderr, "player.c:%s [%s]\n", mesg, info);
    exit(1);
}


int moveon(char *name)
{
    footprint[fppos] = name;
    fppos = ((fppos+1) % RING_SIZE);
    if (fppos == fptail)
        fptail = ((fptail+1) % RING_SIZE);
    return fppos;
}


char *moveback(void)
{
    if (fppos==fptail)
        return NULL;
    fppos = ((fppos-1) % RING_SIZE);
    return footprint[fppos];
}


void help()
{
    printf("\n Available commands:\n");
    printf("\tquit:\tAbandon the game.\n");
    printf("\texit:\tDisplay current position and end game.\n");
    printf("\tpos:\tDisplay current position.\n");
    printf("\toptions:\tDisplay the options.\n");
    if (debug_game) printf("\tprev:\tReturn to previous sketch.\n");
    printf("\thelp:\tDisplay this message.\n");
    printf("\t    When there is only one option, invalid options and commands \n\t    are entered to automatically move to the next sketch.\n");
}
